#include "SpiLib.h"

//typedef unsigned char u8;

#define  RE_SUC   0x01
#define  RE_ERROR 0x00



int SpiLIb::spi_transfer(int fd, unsigned char *txbuf, unsigned char *rxbuf, int bytes)
{
    struct spi_ioc_transfer    xfer[2];
    int    status;

    memset(xfer, 0, sizeof(xfer));

    xfer[0].tx_buf = (__u64)txbuf;
    xfer[0].rx_buf = (__u64)rxbuf;
    xfer[0].len = bytes;

    status = ioctl(fd, SPI_IOC_MESSAGE(1), xfer);
    if (status < 0)
    {
        perror("SPI_IOC_MESSAGE");
        return -1;
    }

    return status;

}

void SpiLIb::spi_master_init(const char *name, int fd)
{
    __u8    mode = 3;
    __u8    lsb, bits;
    //__u32    speed = 30000000;
    __u32      speed = 2000000;
    //__u32    speed = 2000000;
    //__u32    speed = 33000000;

    // SPI_IOC_WR_MODE

    ioctl(fd, SPI_IOC_WR_MODE, &mode);
    ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);

    if (ioctl(fd, SPI_IOC_RD_MODE, &mode) < 0)
    {
        perror("SPI rd_mode");
        return;
    }

    if (ioctl(fd, SPI_IOC_RD_LSB_FIRST, &lsb) < 0)
    {
        perror("SPI rd_lsb_fist");
        return;
    }

    if (ioctl(fd, SPI_IOC_RD_BITS_PER_WORD, &bits) < 0)
    {
        perror("SPI rd bits_per_word");
        return;
    }

    if (ioctl(fd, SPI_IOC_RD_MAX_SPEED_HZ, &speed) < 0)
    {
        perror("SPI rd max_speed_hz");
        return;
    }

    __android_log_print(ANDROID_LOG_INFO, "SPi", "%s: spi mode %d, %d bits %sper word, %d Hz max\n", name, mode, bits, lsb ? "(lsb first) " : "", speed);
    //printf("%s: spi mode %d, %d bits %sper word, %d Hz max\n",
    //    name, mode, bits, lsb ? "(lsb first) " : "", speed);
}

unsigned char SpiLIb::get_crc7(const unsigned char *buff, int len)
{
    unsigned char crc7_accum = 0;
    int i;

    for (i=0;  i < len; i++) {
        crc7_accum =
                crc7_table[(crc7_accum << 1) ^ buff[i]];
    }
    return crc7_accum;
}

int SpiLIb::delay(int x)
{
    // while (--x);
    // std::this_thread::sleep_for(std::chrono::milliseconds(50));
    usleep(50000);
    return 0;
}


void SpiLIb::SendCmdHeader(int fd,u8 *cmd, u8 *rxbuf)
{
    int i=0;
    int retval;

#if defined (CONFIG_ATMEL_SPI_DEBUG)
    printf("tx %1d bytes: ", CMD_HEAD_SIZE);
    for (i = 0; i < CMD_HEAD_SIZE; i++)
    {
        printf(" %02x", cmd[i]);
    }
    printf("\n");
#endif

    /* send five command header */
    for (i=0; i< CMD_HEAD_SIZE; i++)
    {
        retval = spi_transfer(fd, cmd+i, rxbuf+i, 1);
        __android_log_print(ANDROID_LOG_INFO, "SPiCMD", "cmd[%d]=%x,rxbuf[%d]=%x",i,*(cmd+i),i, *(rxbuf+i));
        delay(20);
    }

    cmd[0]=0xaa;    //for response

}

void SpiLIb::RcvINS(int fd, u8 *txbuf, u8 *buf, u8 ins)
{
    int retval;
    int cnt = 1000;
    int count=0;
    /* receive ins */
    INS:
    txbuf[0] = 0xaa;
    delay(20);
    while(cnt--)
    {
        retval = spi_transfer(fd, txbuf, buf, 1);
        __android_log_print(ANDROID_LOG_INFO, "SPiINS", "txbuf=%x,buf=%x", *txbuf,*buf);
        if(*buf == ins)
        {
            return;
            break;
        }
        else
            goto INS;
    }
}

void SpiLIb::RcvLEN(int fd, u8 *txbuf, u8 *buf, u8 len)
{

    int retval;
    /* receive length */
    LEN:
    for(int i=0;i<len;i++){
        txbuf[0]=0xaa;
        retval = spi_transfer(fd, txbuf, buf+i, 1);
        __android_log_print(ANDROID_LOG_INFO, "SPiLEN", "txbuf=%x,rxbuf=%x", *txbuf,*(buf+i));
    }

}

//Rcvdata
void SpiLIb::RcvData(int fd, u8 *txbuf, u8 *buf)
{
    int i;
    int retval;

    /* receive data and crc */
    for(i=0; i<*(buf-1); i++)
    {

        retval = spi_transfer(fd, txbuf, buf+i, 1);
        __android_log_print(ANDROID_LOG_INFO, "SPiDATA", "data[%d]=%x",i, *(buf+i));
    }
}
void SpiLIb::RcvData(int fd, u8 *txbuf, u8 *buf,int len)
{
    int i;
    int retval;

    /* receive data and crc */
    for(i=0; i<len; i++)
    {

        retval = spi_transfer(fd, txbuf, buf+i, 1);
        __android_log_print(ANDROID_LOG_INFO, "SPiDATA", "data[%d]=%x",i, *(buf+i));
    }
}



//RcvSW
void SpiLIb::RcvSW(int fd, u8 *txbuf, u8 *buf, u8 sw)
{
    int i;
    int retval;

    SW90:
    /* receive state word */
    delay(20);
    while(1)
    {
        retval = spi_transfer(fd, txbuf, buf, 1);

        __android_log_print(ANDROID_LOG_INFO, "SPiSW", "txbuf=%x,buf=%x", *txbuf,*buf);

        if(*buf != sw)
        {
            goto SW90;
        }
        break;
    }
    retval = spi_transfer(fd, txbuf, buf+1, 1);
}

void SpiLIb::SendEnd(int fd, u8 *txbuf, u8 *buf)
{
    int retval;
    txbuf[0] = 0xaa;
    retval = spi_transfer(fd, txbuf, buf, 1);
    __android_log_print(ANDROID_LOG_INFO, "SPi_SENDEND", "txbuf=%x,rxbuf=%hhu", *txbuf,*buf);
}

void SpiLIb::SendId(int fd, u8 *txbuf, u8 *buf, u8 id)
{

    int retval;
    txbuf[0]=id;
    retval = spi_transfer(fd, txbuf, buf, 1);
    __android_log_print(ANDROID_LOG_INFO, "SPi_SENDID", "txbuf=%x,rxbuf=%hhu", *txbuf,*buf);
}

void SpiLIb::SendData(int fd,u8 *data, u8 *rxbuf,int data_size)
{
    int i=0;
    int retval;
    unsigned char crc[1];
    crc[0]= get_crc7(data,data_size);

    for (i=0; i< data_size; i++)
    {
        retval = spi_transfer(fd, data+i, rxbuf+i, 1);
        __android_log_print(ANDROID_LOG_INFO, "SPi_SENDDATA", "i=%d,txbuf=%x,rxbuf=%x", i,*(data+i),*(rxbuf+i));
        delay(20);
    }
    retval = spi_transfer(fd, crc, rxbuf, 1);

}


int SpiLIb::Spirandom()
{
    int i;
    int cnt;
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";


    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0");
    // devname = "/dev/spidevSE";

    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);

    CMD_RESEND:

    memcpy(txbuf, (const void *) Random_CMD, sizeof(Random_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);     // 指令

    RcvLEN(fd, txbuf,rxbuf+1, txbuf[4]+1);  //长度 多加一个字节的 CRC

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //计算接收到数据的CRC
    if(get_crc7(rxbuf+2, rxbuf[1]-1) != rxbuf[rxbuf[1]+1])
    {
        //CRC Error  命令重发，超过3次，结束
        if(cnt<3)
        {
            cnt++;
            goto CMD_RESEND;
            printf("cnt over\n");
        }
        else
        {
            printf("ERROR\n");
        }
    }

    printf("rx %1d bytes: ", rxbuf[1]+4);
    __android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
    close(fd);

    std::string result = "Random: ";
    char output[16] = { 0 };
    for (i = 0; i < rxbuf[1]+4; i++) {
        sprintf(output, " %02x ", rxbuf[i]);
        result += output;
    }
    __android_log_print(ANDROID_LOG_INFO, "SPi", "%s", result.c_str());
    // printf("\n");

    return 0;
}

int SpiLIb::Version()
{
    int i;
    int cnt;
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    //const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0");

    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

    CMD_RESEND:

    memcpy(txbuf, Version_CMD, sizeof(Version_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);     // 指令

    RcvLEN(fd,txbuf,rxbuf+1, txbuf[4]);  //长度 多加一个字节的 CRC

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //计算接收到数据的CRC
    if(get_crc7(rxbuf+2, rxbuf[1]-1) != rxbuf[rxbuf[1]+1])
    {
        //CRC Error  命令重发，超过3次，结束
        if(cnt<3)
        {
            cnt++;
            goto CMD_RESEND;
            printf("cnt over\n");
        }
        else
        {
            printf("ERROR\n");
        }
    }

    //printf("rx %1d bytes: ", rxbuf[1]+4);
    __android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
    std::string result = "Version: ";
    char output[16] = { 0 };
    for (i = 0; i < rxbuf[1]+4; i++) {
        sprintf(output, " %c ", rxbuf[i]);
        result += output;
    }
    __android_log_print(ANDROID_LOG_INFO, "SPi", "%s", result.c_str());
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "%s", rxbuf);
    // printf("\n");

    close(fd);
    return 0;
}

int SpiLIb::SM2keypair(int index)//产生密钥
{
    int i;
    int cnt;
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0");

    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    SM2Keypair_CMD[3]=index;

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2Keypair_CMD, sizeof(SM2Keypair_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvSW(fd, txbuf, rxbuf, 0x90);


    close(fd);
    return 0;
}

int SpiLIb::SM2OutPub(int index,unsigned char result[])//导出公钥
{
    int i;
    int cnt;
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
   //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0");

    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    SM2OutPub_CMD[3]=index;


    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2OutPub_CMD, sizeof(SM2OutPub_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);     // 指令

    RcvLEN(fd,txbuf,rxbuf+1, txbuf[4]);  //长度 多加一个字节的 CRC

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //计算接收到数据的CRC
    if(get_crc7(rxbuf+2, rxbuf[1]-1) != rxbuf[rxbuf[1]+1])
    {
        //CRC Error  命令重发，超过3次，结束
        if(cnt<3)
        {
            cnt++;
            goto CMD_RESEND;
            printf("cnt over\n");
        }
        else
        {
            printf("ERROR\n");
        }
    }

    //printf("rx %1d bytes: ", rxbuf[1]+4);
    __android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
    //std::string result = "SM2公钥: ";
    //char output[16] = { 0 };
    for (i = 2; i < rxbuf[1]+2; i++) {
        //sprintf(output, " %02x ", rxbuf[i]);
        result[i-2]= rxbuf[i];
    }
    close(fd);
    return 0;
}

int SpiLIb::SM2OutPri(int index,unsigned char result[])//导出私钥
{
    int i;
    int cnt;
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0");

    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    SM2OutPri_CMD[3]=index;

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2OutPri_CMD, sizeof(SM2OutPri_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);     // 指令

    RcvLEN(fd,txbuf,rxbuf+1, txbuf[4]);  //长度 多加一个字节的 CRC

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //计算接收到数据的CRC
    if(get_crc7(rxbuf+2, rxbuf[1]-1) != rxbuf[rxbuf[1]+1])
    {
        //CRC Error  命令重发，超过3次，结束
        if(cnt<3)
        {
            cnt++;
            goto CMD_RESEND;
            printf("cnt over\n");
        }
        else
        {
            printf("ERROR\n");
        }
    }

    //printf("rx %1d bytes: ", rxbuf[1]+4);
    __android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);

    for (i = 2; i < rxbuf[1]+2; i++) {
        //sprintf(output, " %02x ", rxbuf[i]);
        result[i-2]= rxbuf[i];
    }
//    std::string result = "OutPri";
//    char output[16] = { 0 };
//    for (i = 0; i < rxbuf[1]+4; i++) {
//        sprintf(output, " %c ", rxbuf[i]);
//        result += output;
//    }
//    __android_log_print(ANDROID_LOG_INFO, "SPi", "%s", result.c_str());
//    //__android_log_print(ANDROID_LOG_INFO, "SPi", "%s", rxbuf);
//    // printf("\n");

    close(fd);
    return 0;
}

int SpiLIb::SM2InPub(int index,const unsigned char new_key[])//外部公钥导入存放在01
{
    int i;
    int cnt;
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0")
    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    SM2InPub_CMD[3]=index;

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2InPub_CMD, sizeof(SM2InPub_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);     // 指令

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, new_key, 64);

    SendData(fd, txbuf, rxbuf,64);

    SendEnd(fd,txbuf,rxbuf);

    RcvSW(fd, txbuf, rxbuf+1, 0x90);

    std::string result = "InPub: success";
    __android_log_print(ANDROID_LOG_INFO, "SPi", "%s", result.c_str());
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "%s", rxbuf);
    // printf("\n");

    close(fd);
    return 0;
}

int SpiLIb::SM2InPri(int index,const unsigned char new_key[])//导入私钥 没测试
{
    int i;
    int cnt;
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0");

    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    SM2InPri_CMD[3]=index;

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2InPri_CMD, sizeof(SM2InPri_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);     // 指令

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, new_key, 32);

    SendData(fd, txbuf, rxbuf,32);

    SendEnd(fd,txbuf,rxbuf);

    RcvSW(fd, txbuf, rxbuf+1, 0x90);

    std::string result = "InPri: success";
    __android_log_print(ANDROID_LOG_INFO, "SPi", "%s", result.c_str());

    close(fd);
    return 0;
}

int SpiLIb::SM3Hash(unsigned char *to_hash ,int len,unsigned char *out_hash)//原始哈希 跑通了，没有验证
{
    int i;
    int cnt;
    unsigned char txbuf[512];
    unsigned char rxbuf[512];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0")
    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    int re[4]={0}; // 余数数组
    int x = 0; // 余数数组下标
    int a=len;
    while (a != 0) {
        re[x] = a % 16;
        a /= 16;
        x++;
    }
    SM3Hash_CMD[3]=re[2]+re[3]*16;
    SM3Hash_CMD[4]=re[0]+re[1]*16;
    __android_log_print(ANDROID_LOG_INFO, "len", "len=%x", len);
    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM3Hash_CMD, sizeof(SM3Hash_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, to_hash, len);

    SendData(fd, txbuf, rxbuf,len);

    SendEnd(fd,txbuf,rxbuf);

    memcpy(txbuf, (const void *) SM3Hash_CMD, sizeof(SM3Hash_CMD));

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    RcvLEN(fd,txbuf,rxbuf+1, 1);

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
    for (i = 2; i < rxbuf[1]+2; i++) {
        //sprintf(output, " %02x ", rxbuf[i]);
        out_hash[i-2]= rxbuf[i];
    }

    close(fd);
    return 0;
}

int SpiLIb::sm3hash_tosm2(unsigned char *in,int inl,unsigned char *out, unsigned char *pubkey, unsigned char
*pucID, int idl)
{
    int nRet,l;
    unsigned char *Z = NULL;
    int entl = 0;
    unsigned char tmpm[32];
    unsigned char abxy[32 * 4] = {
            0xFF,0xFF,0xFF,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, /* a */
            0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
            0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
            0xFF,0xFC,
            0x28,0xE9,0xFA,0x9E,0x9D,0x9F,0x5E,0x34,0x4D,0x5A,/* b */
            0x9E,0x4B,0xCF,0x65,0x09,0xA7,0xF3,0x97,0x89,0xF5,
            0x15,0xAB,0x8F,0x92,0xDD,0xBC,0xBD,0x41,0x4D,0x94,
            0x0E,0x93,
            0x32,0xC4,0xAE,0x2C,0x1F,0x19,0x81,0x19,0x5F,0x99,/* x */
            0x04,0x46,0x6A,0x39,0xC9,0x94,0x8F,0xE3,0x0B,0xBF,
            0xF2,0x66,0x0B,0xE1,0x71,0x5A,0x45,0x89,0x33,0x4C,
            0x74,0xC7,
            0xBC,0x37,0x36,0xA2,0xF4,0xF6,0x77,0x9C,0x59,0xBD,/* y */
            0xCE,0xE3,0x6B,0x69,0x21,0x53,0xD0,0xA9,0x87,0x7C,
            0xC6,0x2A,0x47,0x40,0x02,0xDF,0x32,0xE5,0x21,0x39,
            0xF0,0xA0
    };
    l = 2 + idl + 32 * 6;
    Z = (unsigned char *)malloc(l);
    if (!Z)
        return -1;
    entl = idl * 8;
    memset(Z + 1, entl & 0xFF, 1);
    entl >>= 8;
    memset(Z, entl & 0xFF, 1);
    memcpy(Z + 2, pucID, idl);
    memcpy(Z + 2 + idl, abxy, 32 *4);
    memcpy(Z + 2 + idl + 4 * 32, pubkey, 32);
    memcpy(Z + 2 + idl + 5 * 32, pubkey+32, 32);
    nRet = SM3Hash(Z,l,tmpm);
    if (nRet != 0)
        goto quit;
    free(Z);
    l = inl + 32;
    Z = (unsigned char *)malloc(l);
    if (!Z) {
        nRet = -1;
        goto quit;
    }
    memcpy(Z,tmpm,32);
    memcpy(Z+32, in, inl);
    nRet = SM3Hash(Z,l,out);
    quit:
    if (Z)
        free(Z);
    return nRet;
}//用于签名的处理哈希值,用户标识01*16

int SpiLIb::SM2Sign(int index,const unsigned char *to_sign,unsigned char *out_sign)//SM2签名 所使用的哈希值应该来源于sm3hash_tosm2
{
    int i;
    int cnt;
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0")
    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);
    SM2Sign_CMD[3]=index;

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2Sign_CMD, sizeof(SM2Sign_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, to_sign, 32);

    SendData(fd, txbuf, rxbuf,32);

    SendEnd(fd,txbuf,rxbuf);

    memcpy(txbuf, (const void *) SM2Sign_CMD, sizeof(SM2Sign_CMD));

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    RcvLEN(fd,txbuf,rxbuf+1, 1);

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    __android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
    for (i = 2; i < rxbuf[1]+2; i++) {
        //sprintf(output, " %02x ", rxbuf[i]);
        out_sign[i-2]= rxbuf[i];
    }

    close(fd);
    return 0;
}

int SpiLIb::SM2VerifySign(int index, unsigned char *hash,unsigned char *vs)//SM2验签
{
    unsigned char txbuf[256];
    unsigned char rxbuf[256];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0")
    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);
    SM2VerifySign_CMD[3]=index;

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2VerifySign_CMD, sizeof(SM2VerifySign_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, hash, 32);
    memcpy(txbuf+32,vs,64);

    SendData(fd, txbuf, rxbuf,96);

    SendEnd(fd,txbuf,rxbuf);

    RcvSW(fd, txbuf, rxbuf, 0x90);

    __android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);


    close(fd);
    return 0;
}

int SpiLIb::SM2encrypt(int index,unsigned char *to_encrypt ,unsigned char *out_encrypt)//加密
{
    int i;
    int cnt;
    unsigned char txbuf[512];
    unsigned char rxbuf[512];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0")
    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);
    SM2encrypt_CMD[3]=index;

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2encrypt_CMD, sizeof(SM2encrypt_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, to_encrypt, 32);

    SendData(fd, txbuf, rxbuf,32);

    SendEnd(fd,txbuf,rxbuf);

    memcpy(txbuf, (const void *) SM2encrypt_CMD, sizeof(SM2encrypt_CMD));

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    RcvLEN(fd,txbuf,rxbuf+1, 1);

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
    for (i = 2; i < rxbuf[1]+2; i++) {
        //sprintf(output, " %02x ", rxbuf[i]);
        out_encrypt[i-2]= rxbuf[i];
    }

    close(fd);
    return 0;
}
int SpiLIb::SM2decoder(int index,unsigned char *to_decoder ,unsigned char *out_decoder)//解密
{
    int i;
    int cnt;
    unsigned char txbuf[512];
    unsigned char rxbuf[512];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0")
    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);
    SM2decoder_CMD[3]=index;

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2decoder_CMD, sizeof(SM2decoder_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, to_decoder, 128);

    SendData(fd, txbuf, rxbuf,128);

    SendEnd(fd,txbuf,rxbuf);

    memcpy(txbuf, (const void *) SM2decoder_CMD, sizeof(SM2decoder_CMD));

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    RcvLEN(fd,txbuf,rxbuf+1, 1);

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
    for (i = 2; i < rxbuf[1]+2; i++) {
        //sprintf(output, " %02x ", rxbuf[i]);
        out_decoder[i-2]= rxbuf[i];
    }

    close(fd);
    return 0;
}

int SpiLIb::SM2cert(int type,int index,string cert,unsigned char *out_cert)//证书
{
    int i;
    int cnt;
    unsigned char txbuf[512];
    unsigned char rxbuf[1024];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0")
    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);

    int certlen=cert.length();
    unsigned char to_cert[certlen];
    for(int x=0;x<certlen;x++){
        to_cert[x]=cert[x];
    }
    SM2cert_CMD[2]=type;
    SM2cert_CMD[3]=index;
    SM2cert_CMD[4]=certlen;

    CMD_RESEND:

    memcpy(txbuf, (const void *) SM2cert_CMD, sizeof(SM2cert_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, to_cert, certlen);

    SendData(fd, txbuf, rxbuf,certlen);

    SendEnd(fd,txbuf,rxbuf);

    memcpy(txbuf, (const void *) SM2cert_CMD, sizeof(SM2cert_CMD));

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    RcvLEN(fd,txbuf,rxbuf+1, 2);

    int re[4]={0};
    int x = 0;
    int a=rxbuf[1];
    while (a != 0) {
        re[x] = a % 16;
        a /= 16;
        x++;
    }

    int outlen=re[0]*256+re[1]*4096+rxbuf[2];

    RcvData(fd, txbuf, rxbuf+2,outlen);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);

    for (i = 2; i < outlen+2; i++) {
        //sprintf(output, " %02x ", rxbuf[i]);
        out_cert[i-2]= rxbuf[i];
    }

    close(fd);
    return 0;
}
int SpiLIb::Indentify(unsigned char *to_idt,unsigned char *out_idt)//安全认证
{
    int i;
    int cnt;
    unsigned char txbuf[512];
    unsigned char rxbuf[512];

    int retval;
    int msglen;
    int fd;
    const char *devname = "/dev/spidevSE";
    //const char *devname = "/dev/spidev0.0";
    //const char *devname = "/dev/mtkgpioctrl";

    //  NrsecSpiPort spi("/dev/spidevSE");
    //
    //    // NrsecSpiPort spi("/dev/spidev0.0")
    fd = open(devname, O_RDWR);
    if (fd < 0) {
        perror("open");
        return 1;
    }
    spi_master_init(devname, fd);


    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);
    //__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

    CMD_RESEND:

    memcpy(txbuf, (const void *) Indentify_CMD, sizeof(Indentify_CMD));

    SendCmdHeader(fd, txbuf, rxbuf);

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    SendId(fd,txbuf,rxbuf, 0x55);

    memcpy(txbuf, to_idt, 32);

    SendData(fd, txbuf, rxbuf,32);

    SendEnd(fd,txbuf,rxbuf);

    memcpy(txbuf, (const void *) SM2decoder_CMD, sizeof(SM2decoder_CMD));

    RcvINS(fd,txbuf,rxbuf,txbuf[1]);

    RcvLEN(fd,txbuf,rxbuf+1, 1);

    RcvData(fd, txbuf, rxbuf+2);

    RcvSW(fd, txbuf, rxbuf+2+rxbuf[1], 0x90);

    //__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
    for (i = 2; i < rxbuf[1]+2; i++) {
        //sprintf(output, " %02x ", rxbuf[i]);
        out_idt[i-2]= rxbuf[i];
    }

    close(fd);
    return 0;
}




