#ifndef TESTCOMM_SPILIB_H
#define TESTCOMM_SPILIB_H
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <string>
#include <thread>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <linux/types.h>
#include <linux/spi/spidev.h>

#include <android/log.h>

using namespace std;

#define CMD_HEAD_SIZE           5
using namespace std;
typedef unsigned char u8;

class SpiLIb{
public:
    int Spirandom();
    int Version();
    int Indentify(unsigned char *to_idt,unsigned char *out_idt);
    int SM2keypair(int index);
    int SM2OutPub(int index,unsigned char result[]);
    int SM2OutPri(int index,unsigned char result[]);
    int SM2InPub(int index,const unsigned char new_key[]);
    int SM2InPri(int index,const unsigned char new_key[]);
    int SM3Hash(unsigned char *to_hash ,int len,unsigned char *out_hash);
    int sm3hash_tosm2(unsigned char *in,int inl,unsigned char *out, unsigned char *pubkey, unsigned char
    *pucID, int idl);
    int SM2Sign(int index,const unsigned char *to_sign,unsigned char *out_sign);
    int SM2VerifySign(int index,unsigned char *hash,unsigned char *vs);
    int SM2encrypt(int index,unsigned char *to_encrypt ,unsigned char *out_encrypt);
    int SM2decoder(int index,unsigned char *to_decoder ,unsigned char *out_decoder);
    int SM2cert(int type,int index,string cert,unsigned char *out_cert);




private:
    const unsigned char crc7_table[256]= {
            0x00, 0x09, 0x12, 0x1b, 0x24, 0x2d, 0x36, 0x3f,
            0x48, 0x41, 0x5a, 0x53, 0x6c, 0x65, 0x7e, 0x77,
            0x19, 0x10, 0x0b, 0x02, 0x3d, 0x34, 0x2f, 0x26,
            0x51, 0x58, 0x43, 0x4a, 0x75, 0x7c, 0x67, 0x6e,
            0x32, 0x3b, 0x20, 0x29, 0x16, 0x1f, 0x04, 0x0d,
            0x7a, 0x73, 0x68, 0x61, 0x5e, 0x57, 0x4c, 0x45,
            0x2b, 0x22, 0x39, 0x30, 0x0f, 0x06, 0x1d, 0x14,
            0x63, 0x6a, 0x71, 0x78, 0x47, 0x4e, 0x55, 0x5c,
            0x64, 0x6d, 0x76, 0x7f, 0x40, 0x49, 0x52, 0x5b,
            0x2c, 0x25, 0x3e, 0x37, 0x08, 0x01, 0x1a, 0x13,
            0x7d, 0x74, 0x6f, 0x66, 0x59, 0x50, 0x4b, 0x42,
            0x35, 0x3c, 0x27, 0x2e, 0x11, 0x18, 0x03, 0x0a,
            0x56, 0x5f, 0x44, 0x4d, 0x72, 0x7b, 0x60, 0x69,
            0x1e, 0x17, 0x0c, 0x05, 0x3a, 0x33, 0x28, 0x21,
            0x4f, 0x46, 0x5d, 0x54, 0x6b, 0x62, 0x79, 0x70,
            0x07, 0x0e, 0x15, 0x1c, 0x23, 0x2a, 0x31, 0x38,
            0x41, 0x48, 0x53, 0x5a, 0x65, 0x6c, 0x77, 0x7e,
            0x09, 0x00, 0x1b, 0x12, 0x2d, 0x24, 0x3f, 0x36,
            0x58, 0x51, 0x4a, 0x43, 0x7c, 0x75, 0x6e, 0x67,
            0x10, 0x19, 0x02, 0x0b, 0x34, 0x3d, 0x26, 0x2f,
            0x73, 0x7a, 0x61, 0x68, 0x57, 0x5e, 0x45, 0x4c,
            0x3b, 0x32, 0x29, 0x20, 0x1f, 0x16, 0x0d, 0x04,
            0x6a, 0x63, 0x78, 0x71, 0x4e, 0x47, 0x5c, 0x55,
            0x22, 0x2b, 0x30, 0x39, 0x06, 0x0f, 0x14, 0x1d,
            0x25, 0x2c, 0x37, 0x3e, 0x01, 0x08, 0x13, 0x1a,
            0x6d, 0x64, 0x7f, 0x76, 0x49, 0x40, 0x5b, 0x52,
            0x3c, 0x35, 0x2e, 0x27, 0x18, 0x11, 0x0a, 0x03,
            0x74, 0x7d, 0x66, 0x6f, 0x50, 0x59, 0x42, 0x4b,
            0x17, 0x1e, 0x05, 0x0c, 0x33, 0x3a, 0x21, 0x28,
            0x5f, 0x56, 0x4d, 0x44, 0x7b, 0x72, 0x69, 0x60,
            0x0e, 0x07, 0x1c, 0x15, 0x2a, 0x23, 0x38, 0x31,
            0x46, 0x4f, 0x54, 0x5d, 0x62, 0x6b, 0x70, 0x79
    };
    const unsigned char EK_CMD[5]={0x80,0xd4,0x01,0x00,0x10};
    const unsigned char AK_CMD[5]={0x80,0xd4,0x02,0x00,0x10};
    const unsigned char IV_CMD[5]={0x80,0xd4,0x04,0x00,0x10};
    volatile unsigned char SM1encrpt_CMD[5]={0xa0,0xe0,0x80,0xff,0xff};
    volatile unsigned char SM1decoder_CMD[5]={0xa0,0xe0,0x81,0xff,0xff};
    volatile unsigned char SM2Keypair_CMD[5]={0x80,0xb2,0x00,0xff,0x00};
    volatile unsigned char SM2OutPub_CMD[5]={0x80,0xb8,0x01,0xff,0x40};
    volatile unsigned char SM2OutPri_CMD[5]={0x80,0xb8,0x02,0xff,0x20};
    volatile unsigned char SM2InPub_CMD[5]={0x80,0xba,0x01,0xff,0x40};
    volatile unsigned char SM2InPri_CMD[5]={0x80,0xba,0x02,0xff,0x20};
    volatile unsigned char SM3Hash_CMD[5]={0x80,0xb5,0x00,0xff,0xff};
    volatile unsigned char SM2Sign_CMD[5]={0x80,0xb4,0x00,0xff,0x20};
    volatile unsigned char SM2VerifySign_CMD[5]={0x80,0xb6,0x00,0xff,0x60};
    volatile unsigned char SM2encrypt_CMD[5]={0x80,0xb3,0x01,0xff,0x20};
    volatile unsigned char SM2decoder_CMD[5]={0x80,0xb3,0x81,0xff,0x80};
    volatile unsigned char SM2cert_CMD[5]={0x80,0xb7,0xff,0xff,0xff};
    volatile unsigned char Random_CMD[5]={0x00,0x84,0x00,0x00,0xff};
    const    unsigned char Version_CMD[5]={0x00,0x5b,0x00,0x00,0x40};
    const    unsigned char Indentify_CMD[5]={0x80,0xb3,0x01,0x04,0x20};

    int spi_transfer(int fd, unsigned char *txbuf, unsigned char *rxbuf, int bytes);
    void spi_master_init(const char *name, int fd);
    unsigned char get_crc7(const unsigned char *buff, int len);
    void SendCmdHeader(int fd,u8 *cmd, u8 *rxbuf);
    int delay(int x);
    void RcvINS(int fd, u8 *txbuf, u8 *buf, u8 ins);
    void RcvLEN(int fd, u8 *txbuf, u8 *buf, u8 len);
    void RcvData(int fd, u8 *txbuf, u8 *buf);
    void RcvData(int fd, u8 *txbuf, u8 *buf, int len);
    void RcvSW(int fd, u8 *txbuf, u8 *buf, u8 sw);
    void SendEnd(int fd, u8 *txbuf, u8 *buf);
    void SendId(int fd, u8 *txbuf, u8 *buf, u8 id);
    void SendData(int fd,u8 *data, u8 *rxbuf,int data_size);



};




#endif //TESTCOMM_SPILIB_H
